@@ -129,7 +129,6 @@ struct GdmGreeterLoginWindowPrivate
         char            *timed_login_username;
         guint            timed_login_timeout_id;
 
-        guint            animation_timeout_id;
         guint            sensitize_power_buttons_timeout_id;
 
         guint            login_button_handler_id;
@@ -243,6 +242,8 @@ set_focus (GdmGreeterLoginWindow *login_window)
 
         if (GTK_WIDGET_REALIZED (entry) && ! GTK_WIDGET_HAS_FOCUS (entry)) {
                 gtk_widget_grab_focus (entry);
+        } else if (GTK_WIDGET_REALIZED (login_window->priv->user_chooser) && ! GTK_WIDGET_HAS_FOCUS (login_window->priv->user_chooser)) {
+                gtk_widget_grab_focus (login_window->priv->user_chooser);
         }
 }
 
@@ -836,7 +837,7 @@ gdm_greeter_login_window_request_timed_login (GdmGreeterLoginWindow *login_windo
         if (login_window->priv->dialog_mode != MODE_SELECTION) {
                 reset_dialog (login_window);
         }
-        gdm_user_chooser_widget_set_show_auto_user (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
+        gdm_user_chooser_widget_set_show_user_auto (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
 
         if (!timed_login_already_enabled) {
                 gdm_user_chooser_widget_set_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser),
@@ -1360,6 +1361,13 @@ shutdown_button_clicked (GtkButton             *button,
         do_system_stop (login_window);
 }
 
+static void
+on_users_loaded (GdmUserChooserWidget  *user_chooser,
+                 GdmGreeterLoginWindow *login_window)
+{
+        gdm_chooser_widget_activate_if_one_item (GDM_CHOOSER_WIDGET (login_window->priv->user_chooser));
+}
+
 static void
 on_user_chosen (GdmUserChooserWidget  *user_chooser,
                 GdmGreeterLoginWindow *login_window)
@@ -1367,6 +1375,8 @@ on_user_chosen (GdmUserChooserWidget  *user_chooser,
         char *user_name;
 
         user_name = gdm_user_chooser_widget_get_chosen_user_name (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser));
+        g_debug ("GdmGreeterLoginWindow: user chosen '%s'", user_name);
+
         if (user_name == NULL) {
                 return;
         }
@@ -1575,7 +1585,7 @@ load_theme (GdmGreeterLoginWindow *login_window)
         glade_set_custom_handler ((GladeXMLCustomWidgetHandler) custom_widget_constructor,
                                   login_window);
         login_window->priv->xml = glade_xml_new (GLADEDIR "/" GLADE_XML_FILE,
-                                                 "window-box",
+                                                 "window-frame",
                                                  PACKAGE);
 
         g_assert (login_window->priv->xml != NULL);
@@ -1602,18 +1612,21 @@ load_theme (GdmGreeterLoginWindow *login_window)
                 }
         }
 
-        box = glade_xml_get_widget (login_window->priv->xml, "window-box");
+        box = glade_xml_get_widget (login_window->priv->xml, "window-frame");
         gtk_container_add (GTK_CONTAINER (login_window), box);
 
-        login_window->priv->user_chooser =
-                glade_xml_get_widget (login_window->priv->xml, "user-chooser");
-
+        login_window->priv->user_chooser = glade_xml_get_widget (login_window->priv->xml,
+                                                                 "user-chooser");
         if (login_window->priv->user_chooser == NULL) {
                 g_critical ("Userlist box not found");
         }
 
         gdm_user_chooser_widget_set_show_only_chosen (GDM_USER_CHOOSER_WIDGET (login_window->priv->user_chooser), TRUE);
 
+        g_signal_connect (login_window->priv->user_chooser,
+                          "loaded",
+                          G_CALLBACK (on_users_loaded),
+                          login_window);
         g_signal_connect (login_window->priv->user_chooser,
                           "activated",
                           G_CALLBACK (on_user_chosen),
@@ -1918,7 +1931,7 @@ on_gconf_key_changed (GConfClient           *client,
                         update_banner_message (login_window);
                 }
         } else {
-                g_debug ("Config key not handled: %s", key);
+                g_debug ("GdmGreeterLoginWindow: Config key not handled: %s", key);
         }
 }
 
@@ -1955,7 +1968,7 @@ gdm_greeter_login_window_init (GdmGreeterLoginWindow *login_window)
         gtk_window_set_skip_taskbar_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_set_skip_pager_hint (GTK_WINDOW (login_window), TRUE);
         gtk_window_stick (GTK_WINDOW (login_window));
-        gtk_container_set_border_width (GTK_CONTAINER (login_window), 25);
+        gtk_container_set_border_width (GTK_CONTAINER (login_window), 0);
 
         g_signal_connect (login_window,
                           "window-state-event",
